module pfcoef_qdrt_class
  use pfcoef_param_class
  implicit none
  private
  public :: partial_fraction_coef
  public :: get_pfcoef_qdrt
  real(DP), save :: minimaxerror(nkey)
  real(DP), save :: alp(nkey,0:npmax)
  real(DP), save :: bet(nkey,1:npmax)
  logical,  save :: m_is_initialized = .false.

contains

subroutine get_pfcoef_qdrt(this,m,mpow)
  implicit none
  type(partial_fraction_coef), intent(inout) :: this
  integer, intent(in) :: m
  integer, intent(in) :: mpow
  integer :: ikey
  call new_pfcoef_qdrt
  if (allocated(this%alp)) deallocate(this%alp)
  if (allocated(this%bet)) deallocate(this%bet)
  allocate(this%alp(0:m))
  allocate(this%bet(1:m))
  ikey = mpow + (m-npmin)*npowmax
  this%npow = mpow
  this%n    = m
  this%minimaxerror = minimaxerror(ikey)
  this%alp(0:m) = alp(ikey,0:m)
  this%bet(1:m) = bet(ikey,1:m)
  return
end subroutine

subroutine new_pfcoef_qdrt()
  implicit none
  integer :: ikey,np,npow
  if (m_is_initialized) return
#include "PFcoef_QuadRootN04E01.dat"
#include "PFcoef_QuadRootN04E02.dat"
#include "PFcoef_QuadRootN04E03.dat"
#include "PFcoef_QuadRootN04E04.dat"
#include "PFcoef_QuadRootN04E05.dat"
#include "PFcoef_QuadRootN04E06.dat"
#include "PFcoef_QuadRootN04E07.dat"
#include "PFcoef_QuadRootN04E08.dat"
#include "PFcoef_QuadRootN05E01.dat"
#include "PFcoef_QuadRootN05E02.dat"
#include "PFcoef_QuadRootN05E03.dat"
#include "PFcoef_QuadRootN05E04.dat"
#include "PFcoef_QuadRootN05E05.dat"
#include "PFcoef_QuadRootN05E06.dat"
#include "PFcoef_QuadRootN05E07.dat"
#include "PFcoef_QuadRootN05E08.dat"
#include "PFcoef_QuadRootN06E01.dat"
#include "PFcoef_QuadRootN06E02.dat"
#include "PFcoef_QuadRootN06E03.dat"
#include "PFcoef_QuadRootN06E04.dat"
#include "PFcoef_QuadRootN06E05.dat"
#include "PFcoef_QuadRootN06E06.dat"
#include "PFcoef_QuadRootN06E07.dat"
#include "PFcoef_QuadRootN06E08.dat"
#include "PFcoef_QuadRootN07E01.dat"
#include "PFcoef_QuadRootN07E02.dat"
#include "PFcoef_QuadRootN07E03.dat"
#include "PFcoef_QuadRootN07E04.dat"
#include "PFcoef_QuadRootN07E05.dat"
#include "PFcoef_QuadRootN07E06.dat"
#include "PFcoef_QuadRootN07E07.dat"
#include "PFcoef_QuadRootN07E08.dat"
#include "PFcoef_QuadRootN08E01.dat"
#include "PFcoef_QuadRootN08E02.dat"
#include "PFcoef_QuadRootN08E03.dat"
#include "PFcoef_QuadRootN08E04.dat"
#include "PFcoef_QuadRootN08E05.dat"
#include "PFcoef_QuadRootN08E06.dat"
#include "PFcoef_QuadRootN08E07.dat"
#include "PFcoef_QuadRootN08E08.dat"
#include "PFcoef_QuadRootN09E01.dat"
#include "PFcoef_QuadRootN09E02.dat"
#include "PFcoef_QuadRootN09E03.dat"
#include "PFcoef_QuadRootN09E04.dat"
#include "PFcoef_QuadRootN09E05.dat"
#include "PFcoef_QuadRootN09E06.dat"
#include "PFcoef_QuadRootN09E07.dat"
#include "PFcoef_QuadRootN09E08.dat"
#include "PFcoef_QuadRootN10E01.dat"
#include "PFcoef_QuadRootN10E02.dat"
#include "PFcoef_QuadRootN10E03.dat"
#include "PFcoef_QuadRootN10E04.dat"
#include "PFcoef_QuadRootN10E05.dat"
#include "PFcoef_QuadRootN10E06.dat"
#include "PFcoef_QuadRootN10E07.dat"
#include "PFcoef_QuadRootN10E08.dat"
#include "PFcoef_QuadRootN11E01.dat"
#include "PFcoef_QuadRootN11E02.dat"
#include "PFcoef_QuadRootN11E03.dat"
#include "PFcoef_QuadRootN11E04.dat"
#include "PFcoef_QuadRootN11E05.dat"
#include "PFcoef_QuadRootN11E06.dat"
#include "PFcoef_QuadRootN11E07.dat"
#include "PFcoef_QuadRootN11E08.dat"
#include "PFcoef_QuadRootN12E01.dat"
#include "PFcoef_QuadRootN12E02.dat"
#include "PFcoef_QuadRootN12E03.dat"
#include "PFcoef_QuadRootN12E04.dat"
#include "PFcoef_QuadRootN12E05.dat"
#include "PFcoef_QuadRootN12E06.dat"
#include "PFcoef_QuadRootN12E07.dat"
#include "PFcoef_QuadRootN12E08.dat"
#include "PFcoef_QuadRootN13E01.dat"
#include "PFcoef_QuadRootN13E02.dat"
#include "PFcoef_QuadRootN13E03.dat"
#include "PFcoef_QuadRootN13E04.dat"
#include "PFcoef_QuadRootN13E05.dat"
#include "PFcoef_QuadRootN13E06.dat"
#include "PFcoef_QuadRootN13E07.dat"
#include "PFcoef_QuadRootN13E08.dat"
#include "PFcoef_QuadRootN14E01.dat"
#include "PFcoef_QuadRootN14E02.dat"
#include "PFcoef_QuadRootN14E03.dat"
#include "PFcoef_QuadRootN14E04.dat"
#include "PFcoef_QuadRootN14E05.dat"
#include "PFcoef_QuadRootN14E06.dat"
#include "PFcoef_QuadRootN14E07.dat"
#include "PFcoef_QuadRootN14E08.dat"
#include "PFcoef_QuadRootN15E01.dat"
#include "PFcoef_QuadRootN15E02.dat"
#include "PFcoef_QuadRootN15E03.dat"
#include "PFcoef_QuadRootN15E04.dat"
#include "PFcoef_QuadRootN15E05.dat"
#include "PFcoef_QuadRootN15E06.dat"
#include "PFcoef_QuadRootN15E07.dat"
#include "PFcoef_QuadRootN15E08.dat"
#include "PFcoef_QuadRootN16E01.dat"
#include "PFcoef_QuadRootN16E02.dat"
#include "PFcoef_QuadRootN16E03.dat"
#include "PFcoef_QuadRootN16E04.dat"
#include "PFcoef_QuadRootN16E05.dat"
#include "PFcoef_QuadRootN16E06.dat"
#include "PFcoef_QuadRootN16E07.dat"
#include "PFcoef_QuadRootN16E08.dat"
#include "PFcoef_QuadRootN17E01.dat"
#include "PFcoef_QuadRootN17E02.dat"
#include "PFcoef_QuadRootN17E03.dat"
#include "PFcoef_QuadRootN17E04.dat"
#include "PFcoef_QuadRootN17E05.dat"
#include "PFcoef_QuadRootN17E06.dat"
#include "PFcoef_QuadRootN17E07.dat"
#include "PFcoef_QuadRootN17E08.dat"
#include "PFcoef_QuadRootN18E01.dat"
#include "PFcoef_QuadRootN18E02.dat"
#include "PFcoef_QuadRootN18E03.dat"
#include "PFcoef_QuadRootN18E04.dat"
#include "PFcoef_QuadRootN18E05.dat"
#include "PFcoef_QuadRootN18E06.dat"
#include "PFcoef_QuadRootN18E07.dat"
#include "PFcoef_QuadRootN18E08.dat"
#include "PFcoef_QuadRootN19E01.dat"
#include "PFcoef_QuadRootN19E02.dat"
#include "PFcoef_QuadRootN19E03.dat"
#include "PFcoef_QuadRootN19E04.dat"
#include "PFcoef_QuadRootN19E05.dat"
#include "PFcoef_QuadRootN19E06.dat"
#include "PFcoef_QuadRootN19E07.dat"
#include "PFcoef_QuadRootN19E08.dat"
#include "PFcoef_QuadRootN20E01.dat"
#include "PFcoef_QuadRootN20E02.dat"
#include "PFcoef_QuadRootN20E03.dat"
#include "PFcoef_QuadRootN20E04.dat"
#include "PFcoef_QuadRootN20E05.dat"
#include "PFcoef_QuadRootN20E06.dat"
#include "PFcoef_QuadRootN20E07.dat"
#include "PFcoef_QuadRootN20E08.dat"
#include "PFcoef_QuadRootN21E01.dat"
#include "PFcoef_QuadRootN21E02.dat"
#include "PFcoef_QuadRootN21E03.dat"
#include "PFcoef_QuadRootN21E04.dat"
#include "PFcoef_QuadRootN21E05.dat"
#include "PFcoef_QuadRootN21E06.dat"
#include "PFcoef_QuadRootN21E07.dat"
#include "PFcoef_QuadRootN21E08.dat"
#include "PFcoef_QuadRootN22E01.dat"
#include "PFcoef_QuadRootN22E02.dat"
#include "PFcoef_QuadRootN22E03.dat"
#include "PFcoef_QuadRootN22E04.dat"
#include "PFcoef_QuadRootN22E05.dat"
#include "PFcoef_QuadRootN22E06.dat"
#include "PFcoef_QuadRootN22E07.dat"
#include "PFcoef_QuadRootN22E08.dat"
#include "PFcoef_QuadRootN23E01.dat"
#include "PFcoef_QuadRootN23E02.dat"
#include "PFcoef_QuadRootN23E03.dat"
#include "PFcoef_QuadRootN23E04.dat"
#include "PFcoef_QuadRootN23E05.dat"
#include "PFcoef_QuadRootN23E06.dat"
#include "PFcoef_QuadRootN23E07.dat"
#include "PFcoef_QuadRootN23E08.dat"
#include "PFcoef_QuadRootN24E01.dat"
#include "PFcoef_QuadRootN24E02.dat"
#include "PFcoef_QuadRootN24E03.dat"
#include "PFcoef_QuadRootN24E04.dat"
#include "PFcoef_QuadRootN24E05.dat"
#include "PFcoef_QuadRootN24E06.dat"
#include "PFcoef_QuadRootN24E07.dat"
#include "PFcoef_QuadRootN24E08.dat"
#include "PFcoef_QuadRootN25E01.dat"
#include "PFcoef_QuadRootN25E02.dat"
#include "PFcoef_QuadRootN25E03.dat"
#include "PFcoef_QuadRootN25E04.dat"
#include "PFcoef_QuadRootN25E05.dat"
#include "PFcoef_QuadRootN25E06.dat"
#include "PFcoef_QuadRootN25E07.dat"
#include "PFcoef_QuadRootN25E08.dat"
#include "PFcoef_QuadRootN26E01.dat"
#include "PFcoef_QuadRootN26E02.dat"
#include "PFcoef_QuadRootN26E03.dat"
#include "PFcoef_QuadRootN26E04.dat"
#include "PFcoef_QuadRootN26E05.dat"
#include "PFcoef_QuadRootN26E06.dat"
#include "PFcoef_QuadRootN26E07.dat"
#include "PFcoef_QuadRootN26E08.dat"
#include "PFcoef_QuadRootN27E01.dat"
#include "PFcoef_QuadRootN27E02.dat"
#include "PFcoef_QuadRootN27E03.dat"
#include "PFcoef_QuadRootN27E04.dat"
#include "PFcoef_QuadRootN27E05.dat"
#include "PFcoef_QuadRootN27E06.dat"
#include "PFcoef_QuadRootN27E07.dat"
#include "PFcoef_QuadRootN27E08.dat"
#include "PFcoef_QuadRootN28E01.dat"
#include "PFcoef_QuadRootN28E02.dat"
#include "PFcoef_QuadRootN28E03.dat"
#include "PFcoef_QuadRootN28E04.dat"
#include "PFcoef_QuadRootN28E05.dat"
#include "PFcoef_QuadRootN28E06.dat"
#include "PFcoef_QuadRootN28E07.dat"
#include "PFcoef_QuadRootN28E08.dat"
#include "PFcoef_QuadRootN29E01.dat"
#include "PFcoef_QuadRootN29E02.dat"
#include "PFcoef_QuadRootN29E03.dat"
#include "PFcoef_QuadRootN29E04.dat"
#include "PFcoef_QuadRootN29E05.dat"
#include "PFcoef_QuadRootN29E06.dat"
#include "PFcoef_QuadRootN29E07.dat"
#include "PFcoef_QuadRootN29E08.dat"
#include "PFcoef_QuadRootN30E01.dat"
#include "PFcoef_QuadRootN30E02.dat"
#include "PFcoef_QuadRootN30E03.dat"
#include "PFcoef_QuadRootN30E04.dat"
#include "PFcoef_QuadRootN30E05.dat"
#include "PFcoef_QuadRootN30E06.dat"
#include "PFcoef_QuadRootN30E07.dat"
#include "PFcoef_QuadRootN30E08.dat"
#include "PFcoef_QuadRootN31E01.dat"
#include "PFcoef_QuadRootN31E02.dat"
#include "PFcoef_QuadRootN31E03.dat"
#include "PFcoef_QuadRootN31E04.dat"
#include "PFcoef_QuadRootN31E05.dat"
#include "PFcoef_QuadRootN31E06.dat"
#include "PFcoef_QuadRootN31E07.dat"
#include "PFcoef_QuadRootN31E08.dat"
#include "PFcoef_QuadRootN32E01.dat"
#include "PFcoef_QuadRootN32E02.dat"
#include "PFcoef_QuadRootN32E03.dat"
#include "PFcoef_QuadRootN32E04.dat"
#include "PFcoef_QuadRootN32E05.dat"
#include "PFcoef_QuadRootN32E06.dat"
#include "PFcoef_QuadRootN32E07.dat"
#include "PFcoef_QuadRootN32E08.dat"
  m_is_initialized = .true.
  return
end subroutine

end module
