#!/usr/bin/ruby
#
# Generates SSOR Kernel for 5-D Domainwall operator (D' or D)
#
# $Id: GenMultSSOR.rb,v 1.3 2010/09/10 06:25:10 ishikawa Exp $
#

require 'optparse'
require 'FieldClasses'

state = SiteRestriction.new("","blocknormal")

flag = 0
forw = 1

OptionParser.new {|opt|
  opt.on('-f') {
    puts "!======================="
    puts "! inverse Forward part"
    puts "!======================="
    forw = 1
  }
  opt.on('-b') {
    puts "!======================="
    puts "! inverse Backward part"
    puts "!======================="
    forw = 0
  }
  opt.on('-d') {
    flag = 1
  }
  opt.parse!(ARGV)
}

if (flag == 0) 
  puts "!======================="
  puts "! normal mode"
  puts "!======================="
else
  puts "!======================="
  puts "! dagger mode"
  puts "!======================="
end

if (forw == 0) 
  state.set_mode("back")
  puts "!======================="
  puts "! backward mode"
  puts "!======================="
else
  state.set_mode("forw")
  puts "!======================="
  puts "! forward mode"
  puts "!======================="
end

amu = Array.new(5)
for i in 1..5
   amu[i-1] = i
end

puts "!================================================="

yt = Spinor.new("yt%y")
y  = Spinor.new("y")
w  = Spinor.new("w")

b  = FieldDWSpinor.new("q")
x  = FieldDWSpinor.new("dq")
vu = FieldGauge.new("vu","vector")

##########################################
#  yt = Hop x
##########################################
puts "    yt%y(:,:) = Z0"

for mu in amu.reverse
  puts  "!---------------------------"
  print "!  mu=",mu,"\n"
  puts  "!---------------------------"
  if (flag == 1)
    gmu = -mu
  else
    gmu = +mu
  end

  ##########################################
  #  yt = yt + (1-gamma_mu)U_mu(n) x(n+mu)
  ##########################################
  state.put_if(+mu)
  state.extra_boundary(+mu)
  if (mu != 5)
    puts  y.assign_proj_up(-gmu,x.field_shift(+mu))
    puts  w.assign_mult_u_half(vu.dir(mu).field,y,+1)
    puts yt.accum_proj_low(-gmu,w)
  else
    puts  y.assign_proj_up(-gmu,x.field_shift(+mu))
    puts yt.accum_proj_low(-gmu,y)
  end
  state.put_endif

  ##############################################
  #  yt = yt + (1+gamma_mu)U_mu(n-mu)^+ x(n-mu)
  ##############################################
  state.put_if(-mu)
  state.extra_boundary(-mu)
  if (mu != 5)
    puts  y.assign_proj_up(+gmu,x.field_shift(-mu))
    puts  w.assign_mult_u_half(vu.dir(mu).field_shift(-mu),y,-1)
    puts yt.accum_proj_low(+gmu,w)
  else
    puts  y.assign_proj_up(+gmu,x.field_shift(-mu))
    puts yt.accum_proj_low(+gmu,y)
  end
  state.put_endif

end

####################################################
# x = b - x + kappa * Hop x
####################################################
puts ""
puts "  " + x.field.get_name + "(:,:) = " + b.field.get_name + "(:,:) - " + x.field.get_name + "(:,:) + yt%y(:,:)*kappa"
puts ""

